@@ -56,10 +56,10 @@ def get_user(*, username: str) -> APIResponse:
 
 @requires_access_custom_view("GET", permissions.RESOURCE_USER)
 @format_parameters({"limit": check_limit})
-def get_users(*, limit: int, order_by: str = "id", offset: str | None = None) -> APIResponse:
+def get_users(*, limit: int, order_by: str = "id", offset: int | None = None) -> APIResponse:
     """Get users."""
     security_manager = cast("FabAuthManager", get_auth_manager()).security_manager
-    session = security_manager.get_session
+    session = security_manager.session
     total_entries = session.execute(select(func.count(User.id))).scalar()
     direction = desc if order_by.startswith("-") else asc
     to_replace = {"user_id": "id"}
@@ -88,6 +88,8 @@ def get_users(*, limit: int, order_by: str = "id", offset: str | None = None) ->
 @requires_access_custom_view("POST", permissions.RESOURCE_USER)
 def post_user() -> APIResponse:
     """Create a new user."""
+    if request.json is None:
+        raise BadRequest("Request body is required")
     try:
         data = user_schema.load(request.json)
     except ValidationError as e:
@@ -131,6 +133,8 @@ def post_user() -> APIResponse:
 @requires_access_custom_view("PUT", permissions.RESOURCE_USER)
 def patch_user(*, username: str, update_mask: UpdateMask = None) -> APIResponse:
     """Update a user."""
+    if request.json is None:
+        raise BadRequest("Request body is required")
     try:
         data = user_schema.load(request.json)
     except ValidationError as e:
@@ -208,7 +212,7 @@ def delete_user(*, username: str) -> APIResponse:
         raise NotFound(title="User not found", detail=detail)
 
     user.roles = []  # Clear foreign keys on this user first.
-    security_manager.get_session.delete(user)
-    security_manager.get_session.commit()
+    security_manager.session.delete(user)
+    security_manager.session.commit()
 
     return NoContent, HTTPStatus.NO_CONTENT
