@@ -17,11 +17,13 @@
 from __future__ import annotations
 
 import pytest
+from sqlalchemy import select
 
+from airflow.providers.fab.auth_manager.models import Role
+from airflow.providers.fab.auth_manager.security_manager.override import EXISTING_ROLES
 from airflow.providers.fab.www.api_connexion.exceptions import EXCEPTIONS_LINK_MAP
 from airflow.providers.fab.www.security import permissions
 
-from tests_common.test_utils.compat import ignore_provider_compatibility_error
 from unit.fab.auth_manager.api_endpoints.api_connexion_utils import (
     assert_401,
     create_role,
@@ -30,33 +32,36 @@ from unit.fab.auth_manager.api_endpoints.api_connexion_utils import (
     delete_user,
 )
 
-with ignore_provider_compatibility_error("2.9.0+", __file__):
-    from airflow.providers.fab.auth_manager.models import Role
-    from airflow.providers.fab.auth_manager.security_manager.override import EXISTING_ROLES
-
 pytestmark = pytest.mark.db_test
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture
 def configured_app(minimal_app_for_auth_api):
     app = minimal_app_for_auth_api
-    create_user(
-        app,
-        username="test",
-        role_name="Test",
-        permissions=[
-            (permissions.ACTION_CAN_CREATE, permissions.RESOURCE_ROLE),
-            (permissions.ACTION_CAN_READ, permissions.RESOURCE_ROLE),
-            (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_ROLE),
-            (permissions.ACTION_CAN_DELETE, permissions.RESOURCE_ROLE),
-            (permissions.ACTION_CAN_READ, permissions.RESOURCE_ACTION),
-        ],
-    )
-    create_user(app, username="test_no_permissions", role_name="TestNoPermissions")
-    yield app
+    with app.app_context():
+        create_user(
+            app,
+            username="test",
+            role_name="Test",
+            permissions=[
+                (permissions.ACTION_CAN_CREATE, permissions.RESOURCE_ROLE),
+                (permissions.ACTION_CAN_READ, permissions.RESOURCE_ROLE),
+                (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_ROLE),
+                (permissions.ACTION_CAN_DELETE, permissions.RESOURCE_ROLE),
+                (permissions.ACTION_CAN_READ, permissions.RESOURCE_ACTION),
+            ],
+        )
+        create_user(app, username="test_no_permissions", role_name="TestNoPermissions")
+        yield app
 
-    delete_user(app, username="test")
-    delete_user(app, username="test_no_permissions")
+        delete_user(app, username="test")
+        delete_user(app, username="test_no_permissions")
+        session = app.appbuilder.session
+        existing_roles = set(EXISTING_ROLES)
+        existing_roles.update(["Test", "TestNoPermissions"])
+        roles = session.scalars(select(Role).where(~Role.name.in_(existing_roles))).unique().all()
+        for role in roles:
+            delete_role(app, role.name)
 
 
 class TestRoleEndpoint:
@@ -65,18 +70,6 @@ class TestRoleEndpoint:
         self.app = configured_app
         self.client = self.app.test_client()
 
-    def teardown_method(self):
-        """
-        Delete all roles except these ones.
-        Test and TestNoPermissions are deleted by delete_user above
-        """
-        session = self.app.appbuilder.get_session
-        existing_roles = set(EXISTING_ROLES)
-        existing_roles.update(["Test", "TestNoPermissions"])
-        roles = session.query(Role).filter(~Role.name.in_(existing_roles)).all()
-        for role in roles:
-            delete_role(self.app, role.name)
-
 
 class TestGetRoleEndpoint(TestRoleEndpoint):
     def test_should_response_200(self):
@@ -353,7 +346,7 @@ class TestDeleteRole(TestRoleEndpoint):
         role = create_role(self.app, "mytestrole")
         response = self.client.delete(f"/fab/v1/roles/{role.name}", environ_overrides={"REMOTE_USER": "test"})
         assert response.status_code == 204
-        role_obj = session.query(Role).filter(Role.name == role.name).all()
+        role_obj = session.scalars(select(Role).where(Role.name == role.name)).all()
         assert len(role_obj) == 0
 
     def test_delete_should_respond_404(self):
